@@ -33,15 +33,9 @@
 
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
-#include <gio/gioerror.h>
-#include <gio/gfile.h>
-#include <gio/gdatainputstream.h>
-#include <gio/gdataoutputstream.h>
-#include <gio/gsocketinputstream.h>
-#include <gio/gsocketoutputstream.h>
-#include <gio/gmemoryoutputstream.h>
-#include <gio/gmemoryinputstream.h>
-#include <gio/gcontenttype.h>
+#include <gio/gio.h>
+#include <gio/gunixinputstream.h>
+#include <gio/gunixoutputstream.h>
 
 #include "gvfsbackendsftp.h"
 #include "gvfsjobopenforread.h"
@@ -412,7 +406,7 @@ new_command_stream (GVfsBackendSftp *backend, int type, guint32 *id_out)
   GDataOutputStream *data_stream;
   guint32 id;
 
-  mem_stream = g_memory_output_stream_new (NULL);
+  mem_stream = g_memory_output_stream_new (NULL, 0, (GReallocFunc)g_realloc, NULL);
   data_stream = g_data_output_stream_new (mem_stream);
   g_object_unref (mem_stream);
 
@@ -429,42 +423,47 @@ new_command_stream (GVfsBackendSftp *backend, int type, guint32 *id_out)
   return data_stream;
 }
 
-static GByteArray *
-get_data_from_command_stream (GDataOutputStream *command_stream, gboolean free_data)
+static gpointer
+get_data_from_command_stream (GDataOutputStream *command_stream, gsize *len)
 {
   GOutputStream *mem_stream;
-  GByteArray *array;
+  gpointer data;
   guint32 *len_ptr;
   
   mem_stream = g_filter_output_stream_get_base_stream (G_FILTER_OUTPUT_STREAM (command_stream));
-  g_memory_output_stream_set_free_data (G_MEMORY_OUTPUT_STREAM (mem_stream), free_data);
-  array = g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_stream));
+  data = g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_stream));
+  *len = g_memory_output_stream_get_size (G_MEMORY_OUTPUT_STREAM (mem_stream));
 
-  len_ptr = (guint32 *)array->data;
-  *len_ptr = GUINT32_TO_BE (array->len - 4);
+  len_ptr = (guint32 *)data;
+  *len_ptr = GUINT32_TO_BE (*len - 4);
   
-  return array;
+  return data;
 }
 
 static gboolean
-send_command_sync (GVfsBackendSftp *backend,
-                   GDataOutputStream *command_stream,
-                   GCancellable *cancellable,
-                   GError **error)
+send_command_sync_and_unref_command (GVfsBackendSftp *backend,
+                                     GDataOutputStream *command_stream,
+                                     GCancellable *cancellable,
+                                     GError **error)
 {
-  GByteArray *array;
+  gpointer data;
+  gsize len;
   gsize bytes_written;
   gboolean res;
   
-  array = get_data_from_command_stream (command_stream, TRUE);
+  data = get_data_from_command_stream (command_stream, &len);
 
   res = g_output_stream_write_all (backend->command_stream,
-                                   array->data, array->len,
+                                   data, len,
                                    &bytes_written,
                                    cancellable, error);
   
   if (error == NULL && !res)
     g_warning ("Ignored send_command error\n");
+
+  g_free (data);
+  g_object_unref (command_stream);
+
   return res;
 }
 
@@ -499,9 +498,7 @@ make_reply_stream (guint8 *data, gsize len)
   GInputStream *mem_stream;
   GDataInputStream *data_stream;
   
-  mem_stream = g_memory_input_stream_from_data (data, len);
-  g_memory_input_stream_set_free_data (G_MEMORY_INPUT_STREAM (mem_stream), TRUE);
-
+  mem_stream = g_memory_input_stream_new_from_data (data, len, g_free);
   data_stream = g_data_input_stream_new (mem_stream);
   g_object_unref (mem_stream);
   
@@ -627,8 +624,8 @@ handle_login (GVfsBackend *backend,
   else
     prompt_fd = tty_fd;
 
-  prompt_stream = g_socket_input_stream_new (prompt_fd, FALSE);
-  reply_stream = g_socket_output_stream_new (tty_fd, FALSE);
+  prompt_stream = g_unix_input_stream_new (prompt_fd, FALSE);
+  reply_stream = g_unix_output_stream_new (tty_fd, FALSE);
 
   ret_val = TRUE;
   while (1)
@@ -699,7 +696,7 @@ handle_login (GVfsBackend *backend,
 					    _("Enter password"),
 					    op_backend->user,
 					    NULL,
-					    G_PASSWORD_FLAGS_NEED_PASSWORD,
+					    G_ASK_PASSWORD_NEED_PASSWORD,
 					    &aborted,
 					    &new_password,
 					    NULL,
@@ -847,8 +844,8 @@ read_reply_async (GVfsBackendSftp *backend)
 {
   backend->reply_size_read = 0;
   g_input_stream_read_async (backend->reply_stream,
-			     &backend->reply_size, 4,
-			     0, NULL, read_reply_async_got_len, backend);
+                             &backend->reply_size, 4,
+                             0, NULL, read_reply_async_got_len, backend);
 }
 
 static void send_command (GVfsBackendSftp *backend);
@@ -963,13 +960,14 @@ queue_command_stream_and_free (GVfsBackendSftp *backend,
                                GVfsJob *job,
                                gpointer user_data)
 {
-  GByteArray *array;
+  gpointer data;
+  gsize len;
   DataBuffer *buffer;
-  array = get_data_from_command_stream (command_stream, FALSE);
+
+  data = get_data_from_command_stream (command_stream, &len);
   
-  buffer = data_buffer_new (array->data, array->len);
+  buffer = data_buffer_new (data, len);
   g_object_unref (command_stream);
-  g_byte_array_free (array, FALSE);
   
   expect_reply (backend, id, callback, job, user_data);
   queue_command_buffer (backend, buffer);
@@ -986,8 +984,7 @@ get_uid_sync (GVfsBackendSftp *backend)
   
   command = new_command_stream (backend, SSH_FXP_STAT, NULL);
   put_string (command, ".");
-  send_command_sync (backend, command, NULL, NULL);
-  g_object_unref (command);
+  send_command_sync_and_unref_command (backend, command, NULL, NULL);
 
   reply = read_reply_sync (backend, NULL, NULL);
   if (reply == NULL)
@@ -1053,13 +1050,12 @@ do_mount (GVfsBackend *backend,
   
   g_strfreev (args);
 
-  op_backend->command_stream = g_socket_output_stream_new (stdin_fd, TRUE);
+  op_backend->command_stream = g_unix_output_stream_new (stdin_fd, TRUE);
 
   command = new_command_stream (op_backend, SSH_FXP_INIT, NULL);
   g_data_output_stream_put_int32 (command,
                                   SSH_FILEXFER_VERSION, NULL, NULL);
-  send_command_sync (op_backend, command, NULL, NULL);
-  g_object_unref (command);
+  send_command_sync_and_unref_command (op_backend, command, NULL, NULL);
 
   if (tty_fd == -1)
     res = wait_for_reply (backend, stdout_fd, &error);
@@ -1073,10 +1069,10 @@ do_mount (GVfsBackend *backend,
       return;
     }
 
-  op_backend->reply_stream = g_socket_input_stream_new (stdout_fd, TRUE);
+  op_backend->reply_stream = g_unix_input_stream_new (stdout_fd, TRUE);
 
   make_fd_nonblocking (stderr_fd);
-  is = g_socket_input_stream_new (stderr_fd, TRUE);
+  is = g_unix_input_stream_new (stderr_fd, TRUE);
   op_backend->error_stream = g_data_input_stream_new (is);
   g_object_unref (is);
   
@@ -1130,14 +1126,12 @@ do_mount (GVfsBackend *backend,
   g_vfs_backend_set_mount_spec (backend, sftp_mount_spec);
   g_mount_spec_unref (sftp_mount_spec);
 
+  /* Translators: This is the name of an sftp share, like "sftp on <hostname>" */
   display_name = g_strdup_printf (_("sftp on %s"), op_backend->host);
   g_vfs_backend_set_display_name (backend, display_name);
   g_free (display_name);
   g_vfs_backend_set_icon_name (backend, "folder-remote");
 
-  
-  g_print ("succeeded with sftp mount\n");
-  
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
@@ -1167,7 +1161,7 @@ try_mount (GVfsBackend *backend,
     {
       g_vfs_job_failed (G_VFS_JOB (job),
 			G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-			_("Invalid mount spec"));
+			_("No hostname specified"));
       return TRUE;
     }
 
@@ -1222,6 +1216,7 @@ static gboolean
 error_from_status (GVfsJob *job,
                    GDataInputStream *reply,
                    int failure_error,
+                   int allowed_sftp_error,
                    GError **error)
 {
   guint32 code;
@@ -1233,7 +1228,9 @@ error_from_status (GVfsJob *job,
   
   code = g_data_input_stream_read_uint32 (reply, NULL, NULL);
 
-  if (code == SSH_FX_OK)
+  if (code == SSH_FX_OK ||
+      (allowed_sftp_error != -1 &&
+       code == allowed_sftp_error))
     return TRUE;
 
   if (error)
@@ -1253,11 +1250,12 @@ error_from_status (GVfsJob *job,
 static gboolean
 result_from_status (GVfsJob *job,
                     GDataInputStream *reply,
-                    int failure_error)
+                    int failure_error,
+                    int allowed_sftp_error)
 {
   GError *error;
 
-  if (error_from_status (job, reply, failure_error, &error))
+  if (error_from_status (job, reply, failure_error, allowed_sftp_error, &error))
     {
       g_vfs_job_succeeded (job);
       return TRUE;
@@ -1288,7 +1286,7 @@ parse_attributes (GVfsBackendSftp *backend,
                   GFileInfo *info,
                   const char *basename,
                   GDataInputStream *reply,
-                  GFileAttributeMatcher *attribute_matcher)
+                  GFileAttributeMatcher *matcher)
 {
   guint32 flags;
   GFileType type;
@@ -1296,6 +1294,7 @@ parse_attributes (GVfsBackendSftp *backend,
   guint32 mode;
   gboolean has_uid, free_mimetype;
   char *mimetype;
+  GIcon *icon;
   
   flags = g_data_input_stream_read_uint32 (reply, NULL, NULL);
 
@@ -1303,7 +1302,7 @@ parse_attributes (GVfsBackendSftp *backend,
     g_file_info_set_is_hidden (info, TRUE);
 
   if (basename != NULL && basename[strlen (basename) -1] == '~')
-    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STD_IS_BACKUP, TRUE);
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP, TRUE);
 
   if (flags & SSH_FILEXFER_ATTR_SIZE)
     {
@@ -1377,8 +1376,55 @@ parse_attributes (GVfsBackendSftp *backend,
         }
       
       g_file_info_set_content_type (info, mimetype);
-      g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STD_FAST_CONTENT_TYPE, mimetype);
+      g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, mimetype);
       
+      if (g_file_attribute_matcher_matches (matcher,
+                                            G_FILE_ATTRIBUTE_STANDARD_ICON))
+        {
+          icon = NULL;
+          if (S_ISDIR(mode))
+            icon = g_themed_icon_new ("folder-remote");
+          else if (mimetype)
+            {
+              char *mimetype_icon, *generic_mimetype_icon, *type_icon, *p;
+              char *icon_names[3];
+              int i;
+              
+              mimetype_icon = g_strdup (mimetype);
+              g_strdelimit (mimetype_icon, "/", '-');
+              
+              p = strchr (mimetype, '/');
+              if (p == NULL)
+                p = mimetype + strlen (mimetype);
+              
+              generic_mimetype_icon = g_malloc (p - mimetype + strlen ("-x-generic") + 1);
+              memcpy (generic_mimetype_icon, mimetype, p - mimetype);
+              memcpy (generic_mimetype_icon + (p - mimetype), "-x-generic", strlen ("-x-generic"));
+              generic_mimetype_icon[(p - mimetype) + strlen ("-x-generic")] = 0;
+              
+              type_icon = "text-x-generic";
+              
+              i = 0;
+              icon_names[i++] = mimetype_icon;
+              icon_names[i++] = generic_mimetype_icon;
+              if (strcmp (generic_mimetype_icon, type_icon) != 0 &&
+                  strcmp (mimetype_icon, type_icon) != 0) 
+                icon_names[i++] = type_icon;
+
+              icon = g_themed_icon_new_from_names (icon_names, i);
+              
+              g_free (mimetype_icon);
+              g_free (generic_mimetype_icon);
+              
+            }
+          else
+            icon = g_themed_icon_new ("text-x-generic");
+
+          g_file_info_set_icon (info, icon);
+          g_object_unref (icon);
+        }
+
+
       if (free_mimetype)
         g_free (mimetype);
       
@@ -1402,9 +1448,9 @@ parse_attributes (GVfsBackendSftp *backend,
       char *etag;
       
       v = g_data_input_stream_read_uint32 (reply, NULL, NULL);
-      g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_ACCESS, v);
+      g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_ACCESS, v);
       v = g_data_input_stream_read_uint32 (reply, NULL, NULL);
-      g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, v);
+      g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, v);
 
       etag = g_strdup_printf ("%lu", (long unsigned int)v);
       g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_ETAG_VALUE, etag);
@@ -1431,8 +1477,8 @@ parse_attributes (GVfsBackendSftp *backend,
    * the remote charset encoding
    */
   if (basename != NULL &&
-      g_file_attribute_matcher_matches (attribute_matcher,
-                                        G_FILE_ATTRIBUTE_STD_DISPLAY_NAME))
+      g_file_attribute_matcher_matches (matcher,
+                                        G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME))
     {
       char *display_name = g_filename_display_name (basename);
       
@@ -1447,8 +1493,8 @@ parse_attributes (GVfsBackendSftp *backend,
     }
   
   if (basename != NULL &&
-      g_file_attribute_matcher_matches (attribute_matcher,
-                                        G_FILE_ATTRIBUTE_STD_EDIT_NAME))
+      g_file_attribute_matcher_matches (matcher,
+                                        G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME))
     {
       char *edit_name = g_filename_display_name (basename);
       g_file_info_set_edit_name (info, edit_name);
@@ -1489,7 +1535,7 @@ open_for_read_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, -1);
       return;
     }
 
@@ -1544,7 +1590,7 @@ read_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, SSH_FX_EOF);
       return;
     }
 
@@ -1613,7 +1659,7 @@ seek_read_fstat_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, -1);
       return;
     }
 
@@ -1707,7 +1753,7 @@ close_moved_tempfile (GVfsBackendSftp *backend,
   handle = user_data;
 
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1);
+    result_from_status (job, reply, -1, -1);
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -1736,7 +1782,7 @@ close_deleted_file (GVfsBackendSftp *backend,
   error = NULL;
   res = FALSE;
   if (reply_type == SSH_FXP_STATUS)
-    res = error_from_status (job, reply, -1, &error);
+    res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  _("Invalid reply recieved"));
@@ -1784,7 +1830,7 @@ close_moved_file (GVfsBackendSftp *backend,
   error = NULL;
   res = FALSE;
   if (reply_type == SSH_FXP_STATUS)
-    res = error_from_status (job, reply, -1, &error);
+    res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  _("Invalid reply recieved"));
@@ -1863,7 +1909,7 @@ close_write_reply (GVfsBackendSftp *backend,
   error = NULL;
   res = FALSE;
   if (reply_type == SSH_FXP_STATUS)
-    res = error_from_status (job, reply, -1, &error);
+    res = error_from_status (job, reply, -1, -1, &error);
   else
     g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  _("Invalid reply recieved"));
@@ -1974,7 +2020,7 @@ close_read_reply (GVfsBackendSftp *backend,
   handle = user_data;
 
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1);
+    result_from_status (job, reply, -1, -1);
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -2011,7 +2057,7 @@ create_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, G_IO_ERROR_EXISTS);
+      result_from_status (job, reply, G_IO_ERROR_EXISTS, -1);
       return;
     }
 
@@ -2063,7 +2109,7 @@ append_to_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, -1);
       return;
     }
 
@@ -2138,7 +2184,7 @@ replace_create_temp_reply (GVfsBackendSftp *backend,
   if (reply_type == SSH_FXP_STATUS)
     {
       error = NULL;
-      if (error_from_status (job, reply, G_IO_ERROR_EXISTS, &error))
+      if (error_from_status (job, reply, G_IO_ERROR_EXISTS, -1, &error))
         /* Open should not return OK */
         g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                           _("Invalid reply recieved"));
@@ -2312,7 +2358,7 @@ replace_exclusive_reply (GVfsBackendSftp *backend,
   if (reply_type == SSH_FXP_STATUS)
     {
       error = NULL;
-      if (error_from_status (job, reply, G_IO_ERROR_EXISTS, &error))
+      if (error_from_status (job, reply, G_IO_ERROR_EXISTS, -1, &error))
         /* Open should not return OK */
         g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                           _("Invalid reply recieved"));
@@ -2391,7 +2437,7 @@ write_reply (GVfsBackendSftp *backend,
 
   if (reply_type == SSH_FXP_STATUS)
     {
-      if (result_from_status (job, reply, -1))
+      if (result_from_status (job, reply, -1, -1))
         {
           handle->offset += G_VFS_JOB_WRITE (job)->data_size;
         }
@@ -2450,7 +2496,7 @@ seek_write_fstat_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, -1);
       return;
     }
 
@@ -2575,7 +2621,7 @@ read_dir_got_stat_info (GVfsBackendSftp *backend,
   enum_job = G_VFS_JOB_ENUMERATE (job);
 
   if (g_file_attribute_matcher_matches (enum_job->attribute_matcher,
-                                        G_FILE_ATTRIBUTE_STD_SYMLINK_TARGET))
+                                        G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET))
     {
       data->outstanding_requests++;
       command = new_command_stream (backend,
@@ -2732,7 +2778,7 @@ open_dir_reply (GVfsBackendSftp *backend,
   
   if (reply_type == SSH_FXP_STATUS)
     {
-      result_from_status (job, reply, -1);
+      result_from_status (job, reply, -1, -1);
       return;
     }
 
@@ -2865,7 +2911,7 @@ query_info_stat_reply (GVfsBackendSftp *backend,
   data = job->backend_data;
 
   if (reply_type == SSH_FXP_STATUS)
-    error_from_status (job, reply, -1, &data->stat_error);
+    error_from_status (job, reply, -1, -1, &data->stat_error);
   else if (reply_type != SSH_FXP_ATTRS)
     g_set_error (&data->stat_error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  _("Invalid reply recieved"));
@@ -2896,7 +2942,7 @@ query_info_lstat_reply (GVfsBackendSftp *backend,
   data = job->backend_data;
 
   if (reply_type == SSH_FXP_STATUS)
-    error_from_status (job, reply, -1, &data->lstat_error);
+    error_from_status (job, reply, -1, -1, &data->lstat_error);
   else if (reply_type != SSH_FXP_ATTRS)
     g_set_error (&data->lstat_error, G_IO_ERROR, G_IO_ERROR_FAILED,
                  _("Invalid reply recieved"));
@@ -2971,7 +3017,7 @@ try_query_info (GVfsBackend *backend,
     }
 
   if (g_file_attribute_matcher_matches (job->attribute_matcher,
-                                        G_FILE_ATTRIBUTE_STD_SYMLINK_TARGET))
+                                        G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET))
     {
       data->cb_count++;
       
@@ -2995,7 +3041,7 @@ move_reply (GVfsBackendSftp *backend,
 {
   /* on any unknown error, return NOT_SUPPORTED to get the fallback implementation */
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, G_IO_ERROR_NOT_SUPPORTED); 
+    result_from_status (job, reply, G_IO_ERROR_NOT_SUPPORTED, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3019,6 +3065,8 @@ try_move (GVfsBackend *backend,
                                 &id);
   put_string (command, source);
   put_string (command, destination);
+
+  /* This fails if the target exists, we then fallback to default implementation */
   
   queue_command_stream_and_free (op_backend, command, id, move_reply, G_VFS_JOB (job), NULL);
 
@@ -3034,7 +3082,7 @@ set_display_name_reply (GVfsBackendSftp *backend,
                         gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1); 
+    result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3087,7 +3135,7 @@ make_symlink_reply (GVfsBackendSftp *backend,
                     gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1); 
+    result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3123,7 +3171,7 @@ make_directory_reply (GVfsBackendSftp *backend,
                       gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1); 
+    result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3157,7 +3205,7 @@ delete_remove_reply (GVfsBackendSftp *backend,
                      gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1); 
+    result_from_status (job, reply, -1, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3172,7 +3220,7 @@ delete_rmdir_reply (GVfsBackendSftp *backend,
                     gpointer user_data)
 {
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, G_IO_ERROR_NOT_EMPTY); 
+    result_from_status (job, reply, G_IO_ERROR_NOT_EMPTY, -1); 
   else
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
@@ -3191,7 +3239,7 @@ delete_lstat_reply (GVfsBackendSftp *backend,
   GFileInfo *info;
 
   if (reply_type == SSH_FXP_STATUS)
-    result_from_status (job, reply, -1);
+    result_from_status (job, reply, -1, -1);
   else if (reply_type != SSH_FXP_ATTRS)
     g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                       _("Invalid reply recieved"));
