@@ -55,6 +55,7 @@ struct _GHalVolume {
    */
   GFile *foreign_mount_root;
   GMount *foreign_mount;
+  gboolean is_mountable;
 
   char *name;
   char *icon;
@@ -185,17 +186,17 @@ format_size_for_display (guint64 size)
   if (size < MEGABYTE_FACTOR)
     {
       displayed_size = (double) size / KILOBYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f kB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f kB"), displayed_size);
     } 
   else if (size < GIGABYTE_FACTOR)
     {
       displayed_size = (double) size / MEGABYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f MB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f MB"), displayed_size);
     } 
   else 
     {
       displayed_size = (double) size / GIGABYTE_FACTOR;
-      str = g_strdup_printf (_("%.1f GB Media"), displayed_size);
+      str = g_strdup_printf (_("%.1f GB"), displayed_size);
     }
   
   return str;
@@ -214,9 +215,12 @@ do_update_from_hal (GHalVolume *mv)
   gboolean volume_disc_has_data;
   const char *volume_disc_type;
   gboolean volume_disc_is_blank;
+  const char *volume_fsusage;
+  const char *volume_fstype;
   HalDevice *volume;
   HalDevice *drive;
   char *name;
+  char *size;
 
   volume = mv->device;
   drive = mv->drive_device;
@@ -231,6 +235,8 @@ do_update_from_hal (GHalVolume *mv)
   volume_disc_has_data = hal_device_get_property_bool (volume, "volume.disc.has_data");
   volume_disc_is_blank = hal_device_get_property_bool (volume, "volume.disc.is_blank");
   volume_disc_type = hal_device_get_property_string (volume, "volume.disc.type");
+  volume_fsusage = hal_device_get_property_string (volume, "volume.fsusage");
+  volume_fstype = hal_device_get_property_string (volume, "volume.fstype");
 
   if (volume_is_disc && volume_disc_has_audio && mv->foreign_mount_root != NULL)
     {
@@ -238,19 +244,27 @@ do_update_from_hal (GHalVolume *mv)
     }
   else
     {
-      if (volume_fs_label != NULL && strlen (volume_fs_label) > 0) {
-        name = g_strdup (volume_fs_label);
-      } else if (volume_is_disc) {
-        if (volume_disc_has_audio) {
-          if (volume_disc_has_data)
-            name = g_strdup (_("Mixed Audio/Data Disc"));
-          else
-            name = g_strdup (_("Audio Disc"));
-        } else {
-          name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
-        }
+      if (strcmp (volume_fsusage, "crypto") == 0 && strcmp (volume_fstype, "crypto_LUKS") == 0) {
+        size = format_size_for_display (volume_size);
+        name = g_strdup_printf (_("%s Encrypted Data"), size);
+        g_free (size);
       } else {
-        name = format_size_for_display (volume_size);
+        if (volume_fs_label != NULL && strlen (volume_fs_label) > 0) {
+          name = g_strdup (volume_fs_label);
+        } else if (volume_is_disc) {
+          if (volume_disc_has_audio) {
+            if (volume_disc_has_data)
+              name = g_strdup (_("Mixed Audio/Data Disc"));
+            else
+              name = g_strdup (_("Audio Disc"));
+          } else {
+            name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
+          }
+        } else {
+          size = format_size_for_display (volume_size);
+          name = g_strdup_printf (_("%s Media"), size);
+          g_free (size);
+        }
       }
     }
 
@@ -261,8 +275,50 @@ do_update_from_hal (GHalVolume *mv)
     mv->mount_path = g_strdup (hal_device_get_property_string (volume, "volume.mount_point"));
   else
     mv->mount_path = NULL;
+
+  g_object_set_data_full (G_OBJECT (mv), 
+                          "hal-storage-device-capabilities",
+                          g_strdupv (hal_device_get_property_strlist (mv->drive_device, "info.capabilities")),
+                          (GDestroyNotify) g_strfreev);
+
+  if (volume_disc_type != NULL && strlen (volume_disc_type) == 0)
+    volume_disc_type = NULL;
+  g_object_set_data_full (G_OBJECT (mv), 
+                          "hal-volume.disc.type",
+                          g_strdup (volume_disc_type),
+                          (GDestroyNotify) g_free);
 }
 
+#ifdef _WITH_GPHOTO2
+static void
+do_update_from_hal_for_camera (GHalVolume *v)
+{
+  const char *vendor;
+  const char *product;
+
+  vendor = hal_device_get_property_string (v->drive_device, "usb_device.vendor");
+  product = hal_device_get_property_string (v->drive_device, "usb_device.product");
+
+  if (vendor == NULL)
+    {
+      if (product != NULL)
+        v->name = g_strdup (product);
+      else
+        v->name = g_strdup (_("Camera"));
+    }
+  else
+    {
+      if (product != NULL)
+        v->name = g_strdup_printf ("%s %s", vendor, product);
+      else
+        v->name = g_strdup_printf (_("%s Camera"), vendor);
+    }
+
+  v->icon = g_strdup ("camera");
+  v->mount_path = NULL;
+}
+#endif
+
 static void
 update_from_hal (GHalVolume *mv, gboolean emit_changed)
 {
@@ -277,7 +333,14 @@ update_from_hal (GHalVolume *mv, gboolean emit_changed)
   g_free (mv->name);
   g_free (mv->icon);
   g_free (mv->mount_path);
-  do_update_from_hal (mv);
+#ifdef _WITH_GPHOTO2
+  if (hal_device_has_capability (mv->device, "camera"))
+    do_update_from_hal_for_camera (mv);
+  else
+    do_update_from_hal (mv);
+#else
+    do_update_from_hal (mv);
+#endif
 
   if (emit_changed)
     {
@@ -311,7 +374,7 @@ hal_changed (HalDevice *device, const char *key, gpointer user_data)
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (user_data);
   
-  //g_warning ("hal modifying %s (property %s changed)", hal_volume->device_path, key);
+  /*g_warning ("hal modifying %s (property %s changed)", hal_volume->device_path, key);*/
   update_from_hal (hal_volume, TRUE);
 }
 
@@ -348,28 +411,63 @@ g_hal_volume_new (GVolumeMonitor   *volume_monitor,
                   HalDevice        *device,
                   HalPool          *pool,
                   GFile            *foreign_mount_root,
+                  gboolean          is_mountable,
                   GHalDrive        *drive)
 {
   GHalVolume *volume;
   HalDevice *drive_device;
   const char *storage_udi;
+  const char *device_path;
       
-  storage_udi = hal_device_get_property_string (device, "block.storage_device");
-  if (storage_udi == NULL)
-    return NULL;
+  if (hal_device_has_capability (device, "block"))
+    {
+      storage_udi = hal_device_get_property_string (device, "block.storage_device");
+      if (storage_udi == NULL)
+        return NULL;
       
-  drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
-  if (drive_device == NULL)
-    return NULL;
-  
+      drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
+      if (drive_device == NULL)
+        return NULL;
+      
+      device_path = hal_device_get_property_string (device, "block.device");
+    }
+#ifdef _WITH_GPHOTO2
+  else if (hal_device_has_capability (device, "camera"))
+    {
+      /* OK, so we abuse storage_udi and drive_device for the USB main
+       * device that holds this interface... 
+       */
+      storage_udi = hal_device_get_property_string (device, "info.parent");
+      if (storage_udi == NULL)
+        return NULL;
+      
+      drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
+      if (drive_device == NULL)
+        return NULL;
+
+      /* TODO: other OS'es? Will address this with DK aka HAL 2.0 */
+      device_path = hal_device_get_property_string (drive_device, "linux.device_file");
+      if (strlen (device_path) == 0)
+        device_path = NULL;
+
+      if (foreign_mount_root == NULL)
+        return NULL;
+    }
+#endif
+  else
+    {
+      return NULL;
+    }
+
   volume = g_object_new (G_TYPE_HAL_VOLUME, NULL);
   volume->volume_monitor = volume_monitor;
   g_object_add_weak_pointer (G_OBJECT (volume_monitor), (gpointer) &(volume->volume_monitor));
   volume->mount_path = NULL;
-  volume->device_path = g_strdup (hal_device_get_property_string (device, "block.device"));
+  volume->device_path = g_strdup (device_path);
   volume->device = g_object_ref (device);
   volume->drive_device = g_object_ref (drive_device);
   volume->foreign_mount_root = foreign_mount_root != NULL ? g_object_ref (foreign_mount_root) : NULL;
+  volume->is_mountable = is_mountable;
   
   g_signal_connect_object (device, "hal_property_changed", (GCallback) hal_changed, volume, 0);
   g_signal_connect_object (drive_device, "hal_property_changed", (GCallback) hal_changed, volume, 0);
@@ -474,7 +572,7 @@ static GIcon *
 g_hal_volume_get_icon (GVolume *volume)
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
-  return g_themed_icon_new (hal_volume->icon);
+  return g_themed_icon_new_with_default_fallbacks (hal_volume->icon);
 }
 
 static char *
@@ -494,7 +592,8 @@ g_hal_volume_get_uuid (GVolume *volume)
 static gboolean
 g_hal_volume_can_mount (GVolume *volume)
 {
-  return TRUE;
+  GHalVolume *hal_volume = G_HAL_VOLUME (volume);
+  return hal_volume->is_mountable;
 }
 
 static gboolean
@@ -538,6 +637,15 @@ g_hal_volume_has_mount_path (GHalVolume *volume,
   return FALSE;
 }
 
+gboolean
+g_hal_volume_has_device_path (GHalVolume *volume,
+                              const char  *device_path)
+{
+  if (volume->device_path != NULL)
+    return strcmp (volume->device_path, device_path) == 0;
+  return FALSE;
+}
+
 gboolean
 g_hal_volume_has_udi (GHalVolume  *volume,
                       const char  *udi)
@@ -706,6 +814,11 @@ g_hal_volume_mount (GVolume             *volume,
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
+  /*g_warning ("hal_volume_mount (can_mount=%d foreign=%p device_path=%s)", 
+              g_hal_volume_can_mount (volume),
+              hal_volume->foreign_mount_root,
+              hal_volume->device_path);*/
+
   if (hal_volume->foreign_mount_root != NULL)
     {
       ForeignMountOp *data;
@@ -723,8 +836,11 @@ g_hal_volume_mount (GVolume             *volume,
     }
   else
     {
-      char *argv[] = {"gnome-mount", "-b", "-d", NULL, NULL};
+      char *argv[] = {"gnome-mount", "-b", "-d", NULL, NULL, NULL};
       argv[3] = hal_volume->device_path;
+      /* ask for no dialogs if mount_operation is NULL */
+      if (mount_operation == NULL)
+        argv[4] = "-n";
       spawn_do (volume, cancellable, callback, user_data, argv);
     }
 }
@@ -771,7 +887,7 @@ g_hal_volume_eject (GVolume              *volume,
 {
   GHalVolume *hal_volume = G_HAL_VOLUME (volume);
 
-  g_warning ("hal_volume_eject");
+  /*g_warning ("hal_volume_eject");*/
 
   if (hal_volume->drive != NULL)
     {
